import { dataSource } from "../../../data-source";
import { memberQualifications } from "../../../entity/club/member/memberQualifications";
import DatabaseActionException from "../../../exceptions/databaseActionException";
import InternalException from "../../../exceptions/internalException";
import {
  CreateMemberQualificationCommand,
  DeleteMemberQualificationCommand,
  UpdateMemberQualificationCommand,
} from "./memberQualificationCommand";

export default abstract class MemberQualificationCommandHandler {
  /**
   * @description create memberQualification
   * @param {CreateMemberQualificationCommand} createMemberQualification
   * @returns {Promise<number>}
   */
  static async create(createMemberQualification: CreateMemberQualificationCommand): Promise<number> {
    return await dataSource
      .createQueryBuilder()
      .insert()
      .into(memberQualifications)
      .values({
        note: createMemberQualification.note,
        start: createMemberQualification.start,
        memberId: createMemberQualification.memberId,
        qualificationId: createMemberQualification.qualificationId,
      })
      .execute()
      .then((result) => {
        return result.identifiers[0].id;
      })
      .catch((err) => {
        throw new DatabaseActionException("CREATE", "memberQualification", err);
      });
  }

  /**
   * @description update memberQualification
   * @param {UpdateMemberQualificationCommand} updateMemberQualification
   * @returns {Promise<void>}
   */
  static async update(updateMemberQualification: UpdateMemberQualificationCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .update(memberQualifications)
      .set({
        note: updateMemberQualification.note,
        start: updateMemberQualification.start,
        end: updateMemberQualification.end,
        terminationReason: updateMemberQualification.terminationReason,
        qualificationId: updateMemberQualification.qualificationId,
      })
      .where("id = :id", { id: updateMemberQualification.id })
      .andWhere("memberId = :memberId", { memberId: updateMemberQualification.memberId })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("UPDATE", "memberQualification", err);
      });
  }

  /**
   * @description delete memberQualification
   * @param {DeleteMemberQualificationCommand} deleteMemberQualification
   * @returns {Promise<void>}
   */
  static async delete(deleteMemberQualification: DeleteMemberQualificationCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .delete()
      .from(memberQualifications)
      .where("id = :id", { id: deleteMemberQualification.id })
      .andWhere("memberId = :memberId", { memberId: deleteMemberQualification.memberId })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("DELETE", "memberQualification", err);
      });
  }
}
