import { dataSource } from "../../../data-source";
import { membership } from "../../../entity/club/member/membership";
import DatabaseActionException from "../../../exceptions/databaseActionException";
import InternalException from "../../../exceptions/internalException";
import { CreateMembershipCommand, DeleteMembershipCommand, UpdateMembershipCommand } from "./membershipCommand";

export default abstract class MembershipCommandHandler {
  /**
   * @description create membership
   * @param {CreateMembershipCommand} createMembership
   * @returns {Promise<number>}
   */
  static async create(createMembership: CreateMembershipCommand): Promise<number> {
    let insertId = -1;
    return await dataSource
      .transaction(async (manager) => {
        await manager
          .createQueryBuilder()
          .insert()
          .into(membership)
          .values({
            start: createMembership.start,
            memberId: createMembership.memberId,
            statusId: createMembership.statusId,
          })
          .execute()
          .then((result) => {
            insertId = result.identifiers[0].id;
          });

        await manager
          .createQueryBuilder()
          .update(membership)
          .set({
            end: new Date(new Date(createMembership.start).setDate(new Date(createMembership.start).getDate() - 1)),
            terminationReason: "beendet durch neuen Eintrag.",
          })
          .where("end IS NULL")
          .andWhere("memberId = :memberId", { memberId: createMembership.memberId })
          .andWhere("id <> :id", { id: insertId })
          .execute();
      })
      .then(() => {
        return insertId;
      })
      .catch((err) => {
        throw new DatabaseActionException("CREATE", "membership", err);
      });
  }

  /**
   * @description update membership
   * @param {UpdateMembershipCommand} updateMembership
   * @returns {Promise<void>}
   */
  static async update(updateMembership: UpdateMembershipCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .update(membership)
      .set({
        start: updateMembership.start,
        end: updateMembership.end,
        terminationReason: updateMembership.terminationReason,
        statusId: updateMembership.statusId,
      })
      .where("id = :id", { id: updateMembership.id })
      .andWhere("memberId = :memberId", { memberId: updateMembership.memberId })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("UPDATE", "membership", err);
      });
  }

  /**
   * @description delete membership
   * @param {DeleteMembershipCommand} deleteMembership
   * @returns {Promise<void>}
   */
  static async delete(deleteMembership: DeleteMembershipCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .delete()
      .from(membership)
      .where("id = :id", { id: deleteMembership.id })
      .andWhere("memberId = :memberId", { memberId: deleteMembership.memberId })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("DELETE", "membership", err);
      });
  }
}
