import { dataSource } from "../../../data-source";
import { newsletter } from "../../../entity/club/newsletter/newsletter";
import DatabaseActionException from "../../../exceptions/databaseActionException";
import InternalException from "../../../exceptions/internalException";
import { CreateNewsletterCommand, SendNewsletterCommand, SynchronizeNewsletterCommand } from "./newsletterCommand";

export default abstract class NewsletterCommandHandler {
  /**
   * @description create newsletter
   * @param {CreateNewsletterCommand} createNewsletter
   * @returns {Promise<number>}
   */
  static async create(createNewsletter: CreateNewsletterCommand): Promise<number> {
    return await dataSource
      .createQueryBuilder()
      .insert()
      .into(newsletter)
      .values({
        title: createNewsletter.title,
      })
      .execute()
      .then((result) => {
        return result.identifiers[0].id;
      })
      .catch((err) => {
        throw new DatabaseActionException("CREATE", "newsletter", err);
      });
  }

  /**
   * @description sync newsletter
   * @param {SynchronizeNewsletterCommand} syncNewsletter
   * @returns {Promise<void>}
   */
  static async sync(syncNewsletter: SynchronizeNewsletterCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .update(newsletter)
      .set({
        title: syncNewsletter.title,
        description: syncNewsletter.description,
        newsletterTitle: syncNewsletter.newsletterTitle,
        newsletterText: syncNewsletter.newsletterText,
        newsletterSignatur: syncNewsletter.newsletterSignatur,
        recipientsByQueryId: syncNewsletter.recipientsByQueryId,
      })
      .where("id = :id", { id: syncNewsletter.id })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("SYNC", "newsletter", err);
      });
  }

  /**
   * @description send newsletter
   * @param {SendNewsletterCommand} syncNewsletter
   * @returns {Promise<void>}
   */
  static async send(syncNewsletter: SendNewsletterCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .update(newsletter)
      .set({
        isSent: syncNewsletter.isSent,
      })
      .where("id = :id", { id: syncNewsletter.id })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("SET SEND STATE", "newsletter", err);
      });
  }
}
