import { dataSource } from "../../../data-source";
import { equipment } from "../../../entity/unit/equipment/equipment";
import DatabaseActionException from "../../../exceptions/databaseActionException";
import { CreateEquipmentCommand, UpdateEquipmentCommand, DeleteEquipmentCommand } from "./equipmentCommand";

export default abstract class EquipmentCommandHandler {
  /**
   * @description create equipment
   * @param {CreateEquipmentCommand} createEquipment
   * @returns {Promise<number>}
   */
  static async create(createEquipment: CreateEquipmentCommand): Promise<number> {
    return await dataSource
      .createQueryBuilder()
      .insert()
      .into(equipment)
      .values({
        code: createEquipment.code,
        name: createEquipment.name,
        location: createEquipment.location,
        commissioned: createEquipment.commissioned,
        equipmentTypeId: createEquipment.equipmentTypeId,
      })
      .execute()
      .then((result) => {
        return result.identifiers[0].id;
      })
      .catch((err) => {
        throw new DatabaseActionException("CREATE", "equipment", err);
      });
  }

  /**
   * @description update equipment
   * @param {UpdateEquipmentCommand} updateEquipment
   * @returns {Promise<void>}
   */
  static async update(updateEquipment: UpdateEquipmentCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .update(equipment)
      .set({
        code: updateEquipment.code,
        name: updateEquipment.name,
        location: updateEquipment.location,
        commissioned: updateEquipment.commissioned,
        decommissioned: updateEquipment.decommissioned,
      })
      .where("id = :id", { id: updateEquipment.id })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("UPDATE", "equipment", err);
      });
  }

  /**
   * @description delete equipment
   * @param {DeleteEquipmentCommand} deleteEquipment
   * @returns {Promise<void>}
   */
  static async delete(deleteEquipment: DeleteEquipmentCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .delete()
      .from(equipment)
      .where("id = :id", { id: deleteEquipment.id })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("DELETE", "equipment", err);
      });
  }
}
