import { dataSource } from "../../data-source";
import { repair } from "../../entity/unit/repair";
import DatabaseActionException from "../../exceptions/databaseActionException";
import DamageReportCommandHandler from "./damageReportCommandHandler";
import { CreateRepairCommand, UpdateRepairCommand, DeleteRepairCommand } from "./repairCommand";

export default abstract class RepairCommandHandler {
  /**
   * @description create repair
   * @param {CreateRepairCommand} createRepair
   * @returns {Promise<string>}
   */
  static async create(createRepair: CreateRepairCommand): Promise<string> {
    let resultId = "";
    return await dataSource
      .transaction(async (manager) => {
        await manager
          .createQueryBuilder()
          .insert()
          .into(repair)
          .values({
            status: "in Arbeit",
            title: createRepair.title,
            description: createRepair.description,
            responsible: createRepair.responsible,
            equipmentId: createRepair.affected == "equipment" ? createRepair.affectedId : null,
            vehicleId: createRepair.affected == "vehicle" ? createRepair.affectedId : null,
            wearableId: createRepair.affected == "wearable" ? createRepair.affectedId : null,
          })
          .execute()
          .then((result) => {
            resultId = result.identifiers[0].id;
          });

        for (const report of createRepair.reports) {
          await DamageReportCommandHandler.updateRelatedMaintenanceTransaction(manager, {
            id: report,
            repairId: resultId,
          });
        }
      })
      .then(() => {
        return resultId;
      })
      .catch((err) => {
        throw new DatabaseActionException("CREATE", "repair", err);
      });
  }

  /**
   * @description update repair
   * @param {UpdateRepairCommand} updateRepair
   * @returns {Promise<void>}
   */
  static async update(updateRepair: UpdateRepairCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .update(repair)
      .set({
        status: updateRepair.status,
        title: updateRepair.title,
        description: updateRepair.description,
        reports: updateRepair.reports.map((r) => ({ id: r })),
        responsible: updateRepair.responsible,
      })
      .where("id = :id", { id: updateRepair.id })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("UPDATE", "repair", err);
      });
  }

  /**
   * @description delete repair
   * @param {DeleteRepairCommand} deleteRepair
   * @returns {Promise<void>}
   */
  static async delete(deleteRepair: DeleteRepairCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .delete()
      .from(repair)
      .where("id = :id", { id: deleteRepair.id })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("DELETE", "repair", err);
      });
  }
}
