import { dataSource } from "../../../data-source";
import { vehicle } from "../../../entity/unit/vehicle/vehicle";
import DatabaseActionException from "../../../exceptions/databaseActionException";
import { CreateVehicleCommand, UpdateVehicleCommand, DeleteVehicleCommand } from "./vehicleCommand";

export default abstract class VehicleCommandHandler {
  /**
   * @description create vehicle
   * @param {CreateVehicleCommand} createVehicle
   * @returns {Promise<number>}
   */
  static async create(createVehicle: CreateVehicleCommand): Promise<number> {
    return await dataSource
      .createQueryBuilder()
      .insert()
      .into(vehicle)
      .values({
        code: createVehicle.code,
        name: createVehicle.name,
        location: createVehicle.location,
        commissioned: createVehicle.commissioned,
        vehicleTypeId: createVehicle.vehicleTypeId,
      })
      .execute()
      .then((result) => {
        return result.identifiers[0].id;
      })
      .catch((err) => {
        throw new DatabaseActionException("CREATE", "vehicle", err);
      });
  }

  /**
   * @description update vehicle
   * @param {UpdateVehicleCommand} updateVehicle
   * @returns {Promise<void>}
   */
  static async update(updateVehicle: UpdateVehicleCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .update(vehicle)
      .set({
        code: updateVehicle.code,
        name: updateVehicle.name,
        location: updateVehicle.location,
        commissioned: updateVehicle.commissioned,
      })
      .where("id = :id", { id: updateVehicle.id })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("UPDATE", "vehicle", err);
      });
  }

  /**
   * @description delete vehicle
   * @param {DeleteVehicleCommand} deleteVehicle
   * @returns {Promise<void>}
   */
  static async delete(deleteVehicle: DeleteVehicleCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .delete()
      .from(vehicle)
      .where("id = :id", { id: deleteVehicle.id })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("DELETE", "vehicle", err);
      });
  }
}
