import { dataSource } from "../../../data-source";
import { wearable } from "../../../entity/unit/wearable/wearable";
import DatabaseActionException from "../../../exceptions/databaseActionException";
import { CreateWearableCommand, UpdateWearableCommand, DeleteWearableCommand } from "./wearableCommand";

export default abstract class WearableCommandHandler {
  /**
   * @description create wearable
   * @param {CreateWearableCommand} createWearable
   * @returns {Promise<number>}
   */
  static async create(createWearable: CreateWearableCommand): Promise<number> {
    return await dataSource
      .createQueryBuilder()
      .insert()
      .into(wearable)
      .values({
        code: createWearable.code,
        name: createWearable.name,
        location: createWearable.location,
        commissioned: createWearable.commissioned,
        wearableTypeId: createWearable.wearableTypeId,
        wearerId: createWearable.wearerId,
      })
      .execute()
      .then((result) => {
        return result.identifiers[0].id;
      })
      .catch((err) => {
        throw new DatabaseActionException("CREATE", "wearable", err);
      });
  }

  /**
   * @description update wearable
   * @param {UpdateWearableCommand} updateWearable
   * @returns {Promise<void>}
   */
  static async update(updateWearable: UpdateWearableCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .update(wearable)
      .set({
        code: updateWearable.code,
        name: updateWearable.name,
        location: updateWearable.location,
        commissioned: updateWearable.commissioned,
        decommissioned: updateWearable.decommissioned,
        wearerId: updateWearable.wearerId,
      })
      .where("id = :id", { id: updateWearable.id })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("UPDATE", "wearable", err);
      });
  }

  /**
   * @description delete wearable
   * @param {DeleteWearableCommand} deleteWearable
   * @returns {Promise<void>}
   */
  static async delete(deleteWearable: DeleteWearableCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .delete()
      .from(wearable)
      .where("id = :id", { id: deleteWearable.id })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("DELETE", "wearable", err);
      });
  }
}
