import { Request, Response } from "express";
import CommunicationTypeService from "../../../service/configuration/communicationTypeService";
import CommunicationTypeFactory from "../../../factory/admin/configuration/communicationType";
import CommunicationService from "../../../service/club/member/communicationService";
import {
  CreateCommunicationTypeCommand,
  DeleteCommunicationTypeCommand,
  UpdateCommunicationTypeCommand,
} from "../../../command/configuration/communicationType/communicationTypeCommand";
import CommunicationTypeCommandHandler from "../../../command/configuration/communicationType/communicationTypeCommandHandler";

/**
 * @description get all communicationTypes
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getAllCommunicationTypes(req: Request, res: Response): Promise<any> {
  let types = await CommunicationTypeService.getAll();

  res.json(CommunicationTypeFactory.mapToBase(types));
}

/**
 * @description get communicationType by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getCommunicationTypeById(req: Request, res: Response): Promise<any> {
  const id = parseInt(req.params.id);
  let type = await CommunicationTypeService.getById(id);

  res.json(CommunicationTypeFactory.mapToSingle(type));
}

/**
 * @description get columns available to communicationType
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getCommunicationTypeAvailableColumns(req: Request, res: Response): Promise<any> {
  let columns = CommunicationService.getAvailableColumnsForCommunication();
  res.json(columns);
}

/**
 * @description create new communicationType
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function createCommunicationType(req: Request, res: Response): Promise<any> {
  const communicationType = req.body.communicationType;
  const useColumns = req.body.fields;

  let createCommunicationType: CreateCommunicationTypeCommand = {
    type: communicationType,
    useColumns: useColumns,
  };
  await CommunicationTypeCommandHandler.create(createCommunicationType);

  res.sendStatus(204);
}

/**
 * @description update communicationType
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function updateCommunicationType(req: Request, res: Response): Promise<any> {
  const id = parseInt(req.params.id);
  const communicationType = req.body.communicationType;
  const useColumns = req.body.fields;

  let updateCommunicationType: UpdateCommunicationTypeCommand = {
    id: id,
    type: communicationType,
    useColumns: useColumns,
  };
  await CommunicationTypeCommandHandler.update(updateCommunicationType);

  res.sendStatus(204);
}

/**
 * @description delete communicationType
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function deleteCommunicationType(req: Request, res: Response): Promise<any> {
  const id = parseInt(req.params.id);

  let deleteCommunicationType: DeleteCommunicationTypeCommand = {
    id: id,
  };
  await CommunicationTypeCommandHandler.delete(deleteCommunicationType);

  res.sendStatus(204);
}
