import { Request, Response } from "express";
import EquipmentTypeService from "../../../service/unit/equipment/equipmentTypeService";
import EquipmentTypeFactory from "../../../factory/admin/unit/equipment/equipmentType";
import {
  CreateEquipmentTypeCommand,
  DeleteEquipmentTypeCommand,
  UpdateEquipmentTypeCommand,
} from "../../../command/unit/equipment/equipmentTypeCommand";
import EquipmentTypeCommandHandler from "../../../command/unit/equipment/equipmentTypeCommandHandler";

/**
 * @description get all equipmentTypes
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getAllEquipmentTypes(req: Request, res: Response): Promise<any> {
  let offset = parseInt((req.query.offset as string) ?? "0");
  let count = parseInt((req.query.count as string) ?? "25");
  let search = (req.query.search as string) ?? "";
  let noLimit = req.query.noLimit === "true";

  let [equipmentTypes, total] = await EquipmentTypeService.getAll({ offset, count, search, noLimit });

  res.json({
    equipmentTypes: EquipmentTypeFactory.mapToBase(equipmentTypes),
    total: total,
    offset: offset,
    count: count,
  });
}

/**
 * @description get equipmentType by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getEquipmentTypeById(req: Request, res: Response): Promise<any> {
  const equipmentTypeId = req.params.id;
  let equipmentType = await EquipmentTypeService.getById(equipmentTypeId);

  res.json(EquipmentTypeFactory.mapToSingle(equipmentType));
}

/**
 * @description create equipmentType
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function createEquipmentType(req: Request, res: Response): Promise<any> {
  const type = req.body.type;
  const description = req.body.description;

  let createEquipmentType: CreateEquipmentTypeCommand = {
    type,
    description,
  };
  let equipmentTypeId = await EquipmentTypeCommandHandler.create(createEquipmentType);

  res.status(200).send(equipmentTypeId);
}

/**
 * @description update equipmentType by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function updateEquipmentTypeById(req: Request, res: Response): Promise<any> {
  const equipmentTypeId = req.params.id;
  const type = req.body.type;
  const description = req.body.description;

  let updateEquipmentType: UpdateEquipmentTypeCommand = {
    id: equipmentTypeId,
    type,
    description,
  };
  await EquipmentTypeCommandHandler.update(updateEquipmentType);

  res.sendStatus(204);
}

/**
 * @description delete equipmentType by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function deleteEquipmentTypeById(req: Request, res: Response): Promise<any> {
  const equipmentTypeId = req.params.id;

  let deleteEquipmentType: DeleteEquipmentTypeCommand = {
    id: equipmentTypeId,
  };
  await EquipmentTypeCommandHandler.delete(deleteEquipmentType);

  res.sendStatus(204);
}
