import { Request, Response } from "express";
import InspectionPlanService from "../../../service/unit/inspection/inspectionPlanService";
import InspectionPlanFactory from "../../../factory/admin/unit/inspection/inspectionPlan";
import {
  CreateInspectionPlanCommand,
  DeleteInspectionPlanCommand,
  UpdateInspectionPlanCommand,
} from "../../../command/unit/inspection/inspectionPlanCommand";
import InspectionPlanCommandHandler from "../../../command/unit/inspection/inspectionPlanCommandHandler";
import BadRequestException from "../../../exceptions/badRequestException";
import TypeTester from "../../../helpers/typeTester";
import InspectionPointService from "../../../service/unit/inspection/inspectionPointService";
import InspectionPointFactory from "../../../factory/admin/unit/inspection/inspectionPoint";
import InspectionService from "../../../service/unit/inspection/inspectionService";
import InspectionVersionedPlanCommandHandler from "../../../command/unit/inspection/inspectionVersionedPlanCommandHandler";
import InspectionPointCommandHandler from "../../../command/unit/inspection/inspectionPointCommandHandler";

/**
 * @description get all inspectionPlans
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getAllInspectionPlans(req: Request, res: Response): Promise<any> {
  let offset = parseInt((req.query.offset as string) ?? "0");
  let count = parseInt((req.query.count as string) ?? "25");
  let search = (req.query.search as string) ?? "";
  let noLimit = req.query.noLimit === "true";
  let ids = ((req.query.ids ?? "") as string).split(",").filter((i) => i);

  let [inspectionPlans, total] = await InspectionPlanService.getAll({
    offset,
    count,
    search,
    noLimit,
    ids,
  });

  res.json({
    inspectionPlans: InspectionPlanFactory.mapToBase(inspectionPlans),
    total: total,
    offset: offset,
    count: count,
  });
}

/**
 * @description get all inspectionPlans
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getAllInspectionPlansForRelated(req: Request, res: Response): Promise<any> {
  let relation = req.params.related as "vehicleType" | "equipmentType" | "wearableType";
  let relationTypeId = req.params.relatedTypeId as string;
  let offset = parseInt((req.query.offset as string) ?? "0");
  let count = parseInt((req.query.count as string) ?? "25");
  let search = (req.query.search as string) ?? "";
  let noLimit = req.query.noLimit === "true";
  let ids = ((req.query.ids ?? "") as string).split(",").filter((i) => i);

  let where;
  if (relation == "equipmentType") {
    where = { equipmentTypeId: relationTypeId };
  } else if (relation == "vehicleType") {
    where = { vehicleTypeId: relationTypeId };
  } else {
    where = { wearableTypeId: relationTypeId };
  }
  let [inspectionPlans, total] = await InspectionPlanService.getAllForRelated(where, {
    offset,
    count,
    search,
    noLimit,
    ids,
  });

  res.json({
    inspectionPlans: inspectionPlans,
    total: total,
    offset: offset,
    count: count,
  });
}

/**
 * @description get inspectionPoints by planid
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getInspectionPointsByPlanId(req: Request, res: Response): Promise<any> {
  const inspectionPlanId = req.params.id;
  let inspectionPlan = await InspectionPlanService.getById(inspectionPlanId);

  if (!inspectionPlan.latestVersionedPlan) {
    res.json([]);
  } else {
    let inspectionPoints = await InspectionPointService.getAllForVersionedPlan(inspectionPlan.latestVersionedPlan.id);

    res.json(InspectionPointFactory.mapToBase(inspectionPoints));
  }
}

/**
 * @description get inspectionPlan by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getInspectionPlanById(req: Request, res: Response): Promise<any> {
  const inspectionPlanId = req.params.id;
  let inspectionPlan = await InspectionPlanService.getById(inspectionPlanId);

  res.json(InspectionPlanFactory.mapToSingle(inspectionPlan));
}

/**
 * @description create inspectionPlan
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function createInspectionPlan(req: Request, res: Response): Promise<any> {
  const title = req.body.title;
  const inspectionInterval = req.body.inspectionInterval;
  const remindTime = req.body.remindTime;
  const relatedId = req.body.relatedId;
  const assigned = req.body.assigned;

  TypeTester.testPlanTimeDefinition(inspectionInterval, "inspectionInterval", true);
  TypeTester.testPlanTimeDefinition(remindTime, "remindTime", true);

  if (assigned != "equipment" && assigned != "vehicle" && assigned != "wearable")
    throw new BadRequestException("set assigned to equipment or vehicle or wearable");

  let createInspectionPlan: CreateInspectionPlanCommand = {
    title,
    inspectionInterval,
    remindTime,
    relatedId,
    assigned,
  };
  let inspectionPlanId = await InspectionPlanCommandHandler.create(createInspectionPlan);

  res.status(200).send(inspectionPlanId);
}

/**
 * @description update inspectionPlan by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function updateInspectionPlanById(req: Request, res: Response): Promise<any> {
  const inspectionPlanId = req.params.id;
  const title = req.body.title;
  const inspectionInterval = req.body.inspectionInterval;
  const remindTime = req.body.remindTime;

  TypeTester.testPlanTimeDefinition(inspectionInterval, "inspectionInterval", true);
  TypeTester.testPlanTimeDefinition(remindTime, "remindTime", true);

  let updateInspectionPlan: UpdateInspectionPlanCommand = {
    id: inspectionPlanId,
    title,
    inspectionInterval,
    remindTime,
  };
  await InspectionPlanCommandHandler.update(updateInspectionPlan);

  res.sendStatus(204);
}

/**
 * @description get inspectionPoints by planid
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function updateInspectionPointsByPlanId(req: Request, res: Response): Promise<any> {
  const inspectionPlanId = req.params.id;
  const inspectionPoints = req.body;
  let inspectionPlan = await InspectionPlanService.getById(inspectionPlanId);

  let usedVersionedPlan = inspectionPlan?.latestVersionedPlan?.id
    ? await InspectionService.usesVersionedInspectionPlan(inspectionPlan.latestVersionedPlan.id)
    : true;

  if (usedVersionedPlan) {
    await InspectionVersionedPlanCommandHandler.create({ inspectionPlanId }, inspectionPoints);
  } else {
    await InspectionPointCommandHandler.sync(inspectionPlan.latestVersionedPlan.id, inspectionPoints);
  }

  res.sendStatus(204);
}

/**
 * @description delete inspectionPlan by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function deleteInspectionPlanById(req: Request, res: Response): Promise<any> {
  const inspectionPlanId = req.params.id;

  let deleteInspectionPlan: DeleteInspectionPlanCommand = {
    id: inspectionPlanId,
  };
  await InspectionPlanCommandHandler.delete(deleteInspectionPlan);

  res.sendStatus(204);
}
