import { Request, Response } from "express";
import MaintenanceService from "../../../service/unit/maintenanceService";
import MaintenanceFactory from "../../../factory/admin/unit/maintenance";
import { CreateMaintenanceCommand, UpdateMaintenanceCommand } from "../../../command/unit/maintenanceCommand";
import MaintenanceCommandHandler from "../../../command/unit/maintenanceCommandHandler";
import BadRequestException from "../../../exceptions/badRequestException";

/**
 * @description get all maintenances by status
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getAllMaintenancesByStatus(req: Request, res: Response): Promise<any> {
  let done = req.query.done === "true";
  let offset = parseInt((req.query.offset as string) ?? "0");
  let count = parseInt((req.query.count as string) ?? "25");
  let noLimit = req.query.noLimit === "true";

  let [maintenances, total] = await MaintenanceService.getAll(done, { offset, count, noLimit });

  res.json({
    maintenances: MaintenanceFactory.mapToBase(maintenances),
    total: total,
    offset: offset,
    count: count,
  });
}

/**
 * @description get all maintenances for related id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getAllMaintenancesForRelated(req: Request, res: Response): Promise<any> {
  let relation = req.params.related as "vehicle" | "equipment" | "wearable";
  let relationId = req.params.relatedId as string;
  let offset = parseInt((req.query.offset as string) ?? "0");
  let count = parseInt((req.query.count as string) ?? "25");
  let noLimit = req.query.noLimit === "true";

  let where;
  if (relation == "equipment") {
    where = { equipmentId: relationId };
  } else if (relation == "vehicle") {
    where = { vehicleId: relationId };
  } else {
    where = { wearableId: relationId };
  }
  let [maintenances, total] = await MaintenanceService.getAllForRelated(where, { offset, count, noLimit });

  res.json({
    maintenances: MaintenanceFactory.mapToBase(maintenances),
    total: total,
    offset: offset,
    count: count,
  });
}

/**
 * @description get maintenance by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getMaintenanceById(req: Request, res: Response): Promise<any> {
  const maintenanceId = req.params.id;
  let maintenance = await MaintenanceService.getById(maintenanceId);

  res.json(MaintenanceFactory.mapToSingle(maintenance));
}

/**
 * @description create maintenance
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function createMaintenance(req: Request, res: Response): Promise<any> {
  const description = req.body.description;
  const affectedId = req.body.affectedId;
  const affected = req.body.affected;

  if (affected != "equipment" && affected != "vehicle" && affected != "wearable")
    throw new BadRequestException("set assigned to equipment or vehicle or wearable");

  let createMaintenance: CreateMaintenanceCommand = {
    description,
    affectedId,
    affected,
  };
  let maintenanceId = await MaintenanceCommandHandler.create(createMaintenance);

  res.status(200).send(maintenanceId);
}

/**
 * @description update maintenance by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function updateMaintenanceById(req: Request, res: Response): Promise<any> {
  const maintenanceId = req.params.id;
  const description = req.body.description;
  const status = req.body.status;
  const done = req.body.done;

  let updateMaintenance: UpdateMaintenanceCommand = {
    id: maintenanceId,
    description,
    status,
    done,
  };
  await MaintenanceCommandHandler.update(updateMaintenance);

  res.sendStatus(204);
}
