import { Request, Response } from "express";
import VehicleService from "../../../service/unit/vehicle/vehicleService";
import VehicleFactory from "../../../factory/admin/unit/vehicle/vehicle";
import {
  CreateVehicleCommand,
  DeleteVehicleCommand,
  UpdateVehicleCommand,
} from "../../../command/unit/vehicle/vehicleCommand";
import VehicleCommandHandler from "../../../command/unit/vehicle/vehicleCommandHandler";

/**
 * @description get all vehicles
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getAllVehicles(req: Request, res: Response): Promise<any> {
  let offset = parseInt((req.query.offset as string) ?? "0");
  let count = parseInt((req.query.count as string) ?? "25");
  let search = (req.query.search as string) ?? "";
  let noLimit = req.query.noLimit === "true";
  let ids = ((req.query.ids ?? "") as string).split(",").filter((i) => i);

  let [vehicles, total] = await VehicleService.getAll({ offset, count, search, noLimit, ids });

  res.json({
    vehicles: VehicleFactory.mapToBase(vehicles),
    total: total,
    offset: offset,
    count: count,
  });
}

/**
 * @description get vehicle by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getVehicleById(req: Request, res: Response): Promise<any> {
  const vehicleId = req.params.id;
  let vehicle = await VehicleService.getById(vehicleId);

  res.json(VehicleFactory.mapToSingle(vehicle));
}

/**
 * @description get vehicle by Ids
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getVehiclesByIds(req: Request, res: Response): Promise<any> {
  let ids = req.body.ids as Array<string>;

  let [vehicles, total] = await VehicleService.getAll({ noLimit: true, ids });

  res.json({
    vehicles: VehicleFactory.mapToBase(vehicles),
    total: total,
    offset: 0,
    count: total,
  });
}

/**
 * @description create vehicle
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function createVehicle(req: Request, res: Response): Promise<any> {
  const name = req.body.name;
  const code = req.body.code || null;
  const location = req.body.location;
  const commissioned = req.body.commissioned;
  const vehicleTypeId = req.body.vehicleTypeId;

  let createVehicle: CreateVehicleCommand = {
    code,
    name,
    location,
    commissioned,
    vehicleTypeId,
  };
  let vehicleId = await VehicleCommandHandler.create(createVehicle);

  res.status(200).send(vehicleId);
}

/**
 * @description update vehicle by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function updateVehicleById(req: Request, res: Response): Promise<any> {
  const vehicleId = req.params.id;
  const name = req.body.name;
  const code = req.body.code || null;
  const location = req.body.location;
  const commissioned = req.body.commissioned;
  const decommissioned = req.body.decommissioned || null;

  let updateVehicle: UpdateVehicleCommand = {
    id: vehicleId,
    code,
    name,
    location,
    commissioned,
    decommissioned,
  };
  await VehicleCommandHandler.update(updateVehicle);

  res.sendStatus(204);
}

/**
 * @description delete vehicle by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function deleteVehicleById(req: Request, res: Response): Promise<any> {
  const vehicleId = req.params.id;

  let deleteVehicle: DeleteVehicleCommand = {
    id: vehicleId,
  };
  await VehicleCommandHandler.delete(deleteVehicle);

  res.sendStatus(204);
}
