import { Request, Response } from "express";
import VehicleTypeService from "../../../service/unit/vehicle/vehicleTypeService";
import VehicleTypeFactory from "../../../factory/admin/unit/vehicle/vehicleType";
import {
  CreateVehicleTypeCommand,
  DeleteVehicleTypeCommand,
  UpdateVehicleTypeCommand,
} from "../../../command/unit/vehicle/vehicleTypeCommand";
import VehicleTypeCommandHandler from "../../../command/unit/vehicle/vehicleTypeCommandHandler";

/**
 * @description get all vehicleTypes
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getAllVehicleTypes(req: Request, res: Response): Promise<any> {
  let offset = parseInt((req.query.offset as string) ?? "0");
  let count = parseInt((req.query.count as string) ?? "25");
  let search = (req.query.search as string) ?? "";
  let noLimit = req.query.noLimit === "true";

  let [vehicleTypes, total] = await VehicleTypeService.getAll({ offset, count, search, noLimit });

  res.json({
    vehicleTypes: VehicleTypeFactory.mapToBase(vehicleTypes),
    total: total,
    offset: offset,
    count: count,
  });
}

/**
 * @description get vehicleType by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getVehicleTypeById(req: Request, res: Response): Promise<any> {
  const vehicleTypeId = req.params.id;
  let vehicleType = await VehicleTypeService.getById(vehicleTypeId);

  res.json(VehicleTypeFactory.mapToSingle(vehicleType));
}

/**
 * @description create vehicleType
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function createVehicleType(req: Request, res: Response): Promise<any> {
  const type = req.body.type;
  const description = req.body.description;

  let createVehicleType: CreateVehicleTypeCommand = {
    type,
    description,
  };
  let vehicleTypeId = await VehicleTypeCommandHandler.create(createVehicleType);

  res.status(200).send(vehicleTypeId);
}

/**
 * @description update vehicleType by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function updateVehicleTypeById(req: Request, res: Response): Promise<any> {
  const vehicleTypeId = req.params.id;
  const type = req.body.type;
  const description = req.body.description;

  let updateVehicleType: UpdateVehicleTypeCommand = {
    id: vehicleTypeId,
    type,
    description,
  };
  await VehicleTypeCommandHandler.update(updateVehicleType);

  res.sendStatus(204);
}

/**
 * @description delete vehicleType by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function deleteVehicleTypeById(req: Request, res: Response): Promise<any> {
  const vehicleTypeId = req.params.id;

  let deleteVehicleType: DeleteVehicleTypeCommand = {
    id: vehicleTypeId,
  };
  await VehicleTypeCommandHandler.delete(deleteVehicleType);

  res.sendStatus(204);
}
