import { Request, Response } from "express";
import WearableTypeService from "../../../service/unit/wearable/wearableTypeService";
import WearableTypeFactory from "../../../factory/admin/unit/wearable/wearableType";
import {
  CreateWearableTypeCommand,
  DeleteWearableTypeCommand,
  UpdateWearableTypeCommand,
} from "../../../command/unit/wearable/wearableTypeCommand";
import WearableTypeCommandHandler from "../../../command/unit/wearable/wearableTypeCommandHandler";

/**
 * @description get all wearableTypes
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getAllWearableTypes(req: Request, res: Response): Promise<any> {
  let offset = parseInt((req.query.offset as string) ?? "0");
  let count = parseInt((req.query.count as string) ?? "25");
  let search = (req.query.search as string) ?? "";
  let noLimit = req.query.noLimit === "true";

  let [wearableTypes, total] = await WearableTypeService.getAll({ offset, count, search, noLimit });

  res.json({
    wearableTypes: WearableTypeFactory.mapToBase(wearableTypes),
    total: total,
    offset: offset,
    count: count,
  });
}

/**
 * @description get wearableType by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getWearableTypeById(req: Request, res: Response): Promise<any> {
  const wearableTypeId = req.params.id;
  let wearableType = await WearableTypeService.getById(wearableTypeId);

  res.json(WearableTypeFactory.mapToSingle(wearableType));
}

/**
 * @description create wearableType
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function createWearableType(req: Request, res: Response): Promise<any> {
  const type = req.body.type;
  const description = req.body.description;

  let createWearableType: CreateWearableTypeCommand = {
    type,
    description,
  };
  let wearableTypeId = await WearableTypeCommandHandler.create(createWearableType);

  res.status(200).send(wearableTypeId);
}

/**
 * @description update wearableType by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function updateWearableTypeById(req: Request, res: Response): Promise<any> {
  const wearableTypeId = req.params.id;
  const type = req.body.type;
  const description = req.body.description;

  let updateWearableType: UpdateWearableTypeCommand = {
    id: wearableTypeId,
    type,
    description,
  };
  await WearableTypeCommandHandler.update(updateWearableType);

  res.sendStatus(204);
}

/**
 * @description delete wearableType by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function deleteWearableTypeById(req: Request, res: Response): Promise<any> {
  const wearableTypeId = req.params.id;

  let deleteWearableType: DeleteWearableTypeCommand = {
    id: wearableTypeId,
  };
  await WearableTypeCommandHandler.delete(deleteWearableType);

  res.sendStatus(204);
}
