import { Request, Response } from "express";
import { JWTHelper } from "../helpers/jwtHelper";
import { JWTToken } from "../type/jwtTypes";
import InternalException from "../exceptions/internalException";
import RefreshCommandHandler from "../command/refreshCommandHandler";
import { CreateRefreshCommand, DeleteRefreshCommand } from "../command/refreshCommand";
import UserService from "../service/management/userService";
import speakeasy from "speakeasy";
import UnauthorizedRequestException from "../exceptions/unauthorizedRequestException";
import RefreshService from "../service/refreshService";
import { LoginRoutineEnum } from "../enums/loginRoutineEnum";

/**
 * @description Check authentication status by token
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function kickof(req: Request, res: Response): Promise<any> {
  let username = req.body.username;

  let { routine } = await UserService.getByUsername(username).catch(() => {
    throw new UnauthorizedRequestException("Username not found");
  });

  res.json({
    routine,
  });
}

/**
 * @description Check authentication status by token
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function login(req: Request, res: Response): Promise<any> {
  let username = req.body.username;
  let passedSecret = req.body.secret;

  let { id } = await UserService.getByUsername(username);
  let { secret, routine } = await UserService.getUserSecretAndRoutine(id);

  let valid = false;
  if (routine == LoginRoutineEnum.totp) {
    valid = speakeasy.totp.verify({
      secret: secret,
      encoding: "base32",
      token: passedSecret,
      window: 2,
    });
  } else {
    valid = passedSecret == secret;
  }

  if (!valid) {
    throw new UnauthorizedRequestException("Credentials not valid or expired");
  }

  let accessToken = await JWTHelper.buildToken(id);

  let refreshCommand: CreateRefreshCommand = {
    userId: id,
    isFromPwa: req.isPWA,
  };
  let refreshToken = await RefreshCommandHandler.create(refreshCommand);

  res.json({
    accessToken,
    refreshToken,
  });
}

/**
 * @description logout user by token (invalidate refresh token)
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function logout(req: Request, res: Response): Promise<any> {}

/**
 * @description refresh expired token
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function refresh(req: Request, res: Response): Promise<any> {
  let token = req.body.accessToken;
  let refresh = req.body.refreshToken;

  const tokenUser = await JWTHelper.decode(token);
  if (typeof tokenUser == "string" || !tokenUser) {
    throw new InternalException("process failed");
  }

  let tokenUserId = (tokenUser as JWTToken).userId;

  let { user } = await RefreshService.getByToken(refresh);

  if (tokenUserId != user.id) {
    throw new UnauthorizedRequestException("user not identified with token and refresh");
  }

  let accessToken = await JWTHelper.buildToken(tokenUserId);

  let refreshCommand: CreateRefreshCommand = {
    userId: tokenUserId,
    isFromPwa: req.isPWA,
  };
  let refreshToken = await RefreshCommandHandler.create(refreshCommand);

  let removeToken: DeleteRefreshCommand = {
    userId: tokenUserId,
    token: refresh,
  };
  await RefreshCommandHandler.deleteByToken(removeToken);

  res.json({
    accessToken,
    refreshToken,
  });
}
