import { inspection } from "../../../../entity/unit/inspection/inspection";
import {
  InspectionNextViewModel,
  InspectionRelated,
  InspectionViewModel,
  MinifiedInspectionViewModel,
} from "../../../../viewmodel/admin/unit/inspection/inspection.models";
import EquipmentFactory from "../equipment/equipment";
import VehicleFactory from "../vehicle/vehicle";
import WearableFactory from "../wearable/wearable";
import InspectionPlanFactory from "./inspectionPlan";
import InspectionPointResultFactory from "./inspectionPointResult";
import InspectionVersionedPlanFactory from "./inspectionVersionedPlan";

export default abstract class InspectionFactory {
  /**
   * @description map record to inspection
   * @param {inspection} record
   * @returns {InspectionViewModel}
   */
  public static mapRelated(record: inspection): InspectionRelated {
    let related: InspectionRelated;
    if (record?.equipmentId) {
      related = {
        relatedId: record.equipmentId,
        assigned: "equipment",
        related: EquipmentFactory.mapToSingle(record.equipment),
      };
    } else if (record?.vehicleId) {
      related = {
        relatedId: record.vehicleId,
        assigned: "vehicle",
        related: VehicleFactory.mapToSingle(record.vehicle),
      };
    } else {
      related = {
        relatedId: record.wearableId,
        assigned: "wearable",
        related: WearableFactory.mapToSingle(record.wearable),
      };
    }
    return related;
  }

  /**
   * @description map record to inspection
   * @param {inspection} record
   * @returns {InspectionViewModel}
   */
  public static mapToSingle(record: inspection): InspectionViewModel {
    let related = this.mapRelated(record);

    return {
      id: record.id,
      inspectionPlanId: record.inspectionPlanId,
      inspectionPlan: InspectionPlanFactory.mapToSingle(record.inspectionPlan),
      inspectionVersionedPlanId: record.inspectionVersionedPlanId,
      inspectionVersionedPlan: InspectionVersionedPlanFactory.mapToSingle(record.inspectionVersionedPlan),
      context: record.context,
      created: record.createdAt,
      finished: record?.finishedAt,
      isOpen: record?.finishedAt == undefined,
      nextInspection: record?.nextInspection,
      checks: InspectionPointResultFactory.mapToBase(record.pointResults),
      ...related,
    };
  }

  /**
   * @description map records to inspection
   * @param {Array<inspection>} records
   * @returns {Array<InspectionViewModel>}
   */
  public static mapToBase(records: Array<inspection>): Array<InspectionViewModel> {
    return records.map((r) => this.mapToSingle(r));
  }

  /**
   * @description map record to minified inspection
   * @param {inspection} record
   * @returns {MinifiedInspectionViewModel}
   */
  public static mapToSingleMinified(record: inspection): MinifiedInspectionViewModel {
    let related = this.mapRelated(record);

    return {
      id: record.id,
      inspectionPlanId: record.inspectionPlanId,
      inspectionPlan: InspectionPlanFactory.mapToSingle(record.inspectionPlan),
      context: record.context,
      created: record.createdAt,
      finished: record?.finishedAt,
      isOpen: record?.finishedAt == undefined,
      nextInspection: record?.nextInspection,
      ...related,
    };
  }

  /**
   * @description map records to minified inspection
   * @param {Array<inspection>} records
   * @returns {Array<MinifiedInspectionViewModel>}
   */
  public static mapToBaseMinified(records: Array<inspection>): Array<MinifiedInspectionViewModel> {
    return records.map((r) => this.mapToSingleMinified(r));
  }

  /**
   * @description map record to next inspection
   * @param {inspection} record
   * @returns {InspectionNextViewModel}
   */
  public static mapToSingleNext(record: inspection): InspectionNextViewModel {
    let related = this.mapRelated(record);

    return {
      id: record.id,
      inspectionPlanId: record.inspectionPlanId,
      inspectionPlan: InspectionPlanFactory.mapToSingle(record.inspectionPlan),
      dueDate: record?.nextInspection,
      ...related,
    };
  }

  /**
   * @description map records to next inspection
   * @param {Array<inspection>} records
   * @returns {Array<InspectionNextViewModel>}
   */
  public static mapToBaseNext(records: Array<inspection>): Array<InspectionNextViewModel> {
    return records.map((r) => this.mapToSingleNext(r));
  }
}
