import express, { Request, Response } from "express";
import PermissionHelper from "../../../helpers/permissionHelper";
import multer from "multer";
import {
  createManualBackup,
  downloadBackupFile,
  downloadUploadedBackupFile,
  getGeneratedBackups,
  getUploadedBackups,
  restoreBackupByLocalFile,
  restoreBackupByUploadedFile,
  uploadBackupFile,
} from "../../../controller/admin/management/backupController";
import { FileSystemHelper } from "../../../helpers/fileSystemHelper";

const storage = multer.diskStorage({
  destination: (req, file, cb) => {
    FileSystemHelper.createFolder("uploaded-backup");
    cb(null, "files/uploaded-backup/");
  },
  filename: (req, file, cb) => {
    cb(null, `${new Date().toISOString().split("T")[0]}_${file.originalname}`);
  },
});

const upload = multer({
  storage,
  fileFilter: (req: Request, file, cb) => {
    if (file.mimetype === "application/json") {
      cb(null, true);
    } else {
      cb(new Error("Only JSON files are allowed!"));
    }
  },
});

var router = express.Router({ mergeParams: true });

router.get("/generated", async (req: Request, res: Response) => {
  await getGeneratedBackups(req, res);
});

router.get("/generated/:filename", async (req: Request, res: Response) => {
  await downloadBackupFile(req, res);
});

router.get("/uploaded", async (req: Request, res: Response) => {
  await getUploadedBackups(req, res);
});

router.get("/uploaded/:filename", async (req: Request, res: Response) => {
  await downloadUploadedBackupFile(req, res);
});

router.post(
  "/",
  PermissionHelper.passCheckMiddleware("create", "management", "backup"),
  async (req: Request, res: Response) => {
    await createManualBackup(req, res);
  }
);

router.post(
  "/generated/restore",
  PermissionHelper.passCheckMiddleware("admin", "management", "backup"),
  async (req: Request, res: Response) => {
    await restoreBackupByLocalFile(req, res);
  }
);

router.post(
  "/uploaded/restore",
  PermissionHelper.passCheckMiddleware("admin", "management", "backup"),
  async (req: Request, res: Response) => {
    await restoreBackupByUploadedFile(req, res);
  }
);

router.post(
  "/upload",
  PermissionHelper.passCheckMiddleware("create", "management", "backup"),
  upload.single("file"),
  async (req: Request, res: Response) => {
    await uploadBackupFile(req, res);
  }
);

export default router;
