import { dataSource } from "../../../data-source";
import { communication } from "../../../entity/club/member/communication";
import DatabaseActionException from "../../../exceptions/databaseActionException";
import InternalException from "../../../exceptions/internalException";

export default abstract class CommunicationService {
  /**
   * @description get all by member id
   * @param {string} memberId
   * @returns {Promise<Array<communication>>}
   */
  static async getAll(memberId: string): Promise<Array<communication>> {
    return await dataSource
      .getRepository(communication)
      .createQueryBuilder("communication")
      .leftJoinAndSelect("communication.type", "communicationType")
      .leftJoinAndSelect("communication.member", "member")
      .where("communication.memberId = :memberId", { memberId: memberId })
      .orderBy("communicationType.type", "ASC")
      .getMany()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "memberCommunications", err);
      });
  }

  /**
   * @description get all by memberId and recordId
   * @param {string} memberId
   * @param {number} recordId
   * @returns {Promise<communication>}
   */
  static async getById(memberId: string, recordId: number): Promise<communication> {
    return await dataSource
      .getRepository(communication)
      .createQueryBuilder("communication")
      .leftJoinAndSelect("communication.type", "communicationType")
      .leftJoinAndSelect("communication.member", "member")
      .where("communication.memberId = :memberId", { memberId: memberId })
      .andWhere("communication.id = :recordId", { recordId: recordId })
      .getOneOrFail()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "memberCommunication", err);
      });
  }

  /**
   * @description get available columns for self made com type
   * @returns {Array<string>}
   */
  static getAvailableColumnsForCommunication(): Array<string> {
    let metadata = dataSource.getMetadata(communication);
    let columns = metadata.columns.map((c) => c.propertyName);
    return columns.filter(
      (c) =>
        !["id", "preferred", "isSMSAlarming", "isSendNewsletter", "typeId", "memberId", "type", "member"].includes(c)
    );
  }
}
