import { Brackets, ILike, Not, SelectQueryBuilder } from "typeorm";
import { dataSource } from "../../../data-source";
import { member } from "../../../entity/club/member/member";
import DatabaseActionException from "../../../exceptions/databaseActionException";
import { memberView } from "../../../views/memberView";

export default abstract class MemberService {
  /**
   * @description get all members
   * @returns {Promise<[Array<member>, number]>}
   */
  static async getAll({
    offset = 0,
    count = 25,
    search = "",
    noLimit = false,
    ids = [],
  }: {
    offset?: number;
    count?: number;
    search?: string;
    noLimit?: boolean;
    ids?: Array<string>;
  }): Promise<[Array<member>, number]> {
    let query = this.applyMemberBaseJoins();

    if (search != "") {
      let searchBits = search.split(" ");

      if (searchBits.length < 2) {
        query = query.where(
          new Brackets((qb) =>
            qb
              .where({ firstname: ILike(`%${searchBits[0]}%`) })
              .orWhere({ lastname: ILike(`%${searchBits[0]}%`) })
              .orWhere({ internalId: ILike(`%${searchBits[0]}%`) })
          )
        );
      } else {
        searchBits
          .flatMap((v, i) => searchBits.slice(i + 1).map((w) => [v, w]))
          .forEach((term) => {
            query = query
              .orWhere(
                new Brackets((qb) =>
                  qb.where({ firstname: ILike(`%${term[0]}%`) }).andWhere({ lastname: ILike(`%${term[1]}%`) })
                )
              )
              .orWhere(
                new Brackets((qb) =>
                  qb.where({ firstname: ILike(`%${term[1]}%`) }).andWhere({ lastname: ILike(`%${term[0]}%`) })
                )
              );
          });
      }
    }

    if (ids.length != 0) {
      query = query.where("member.id IN (:...ids)", { ids: ids });
    }

    if (!noLimit) {
      query = query.offset(offset).limit(count);
    }

    return await query
      .orderBy("member.lastname")
      .addOrderBy("member.firstname")
      .addOrderBy("member.nameaffix")
      .getManyAndCount()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "member", err);
      });
  }

  /**
   * @description get member by id
   * @param {string} id
   * @returns {Promise<member>}
   */
  static async getById(id: string): Promise<member> {
    return this.applyMemberBaseJoins()
      .where("member.id = :id", { id: id })
      .getOneOrFail()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "member", err);
      });
  }

  /**
   * @description get member statistics by id
   * @param {string} id
   * @returns {Promise<memberView>}
   */
  static async getStatisticsById(id: string): Promise<memberView> {
    return await dataSource
      .getRepository(memberView)
      .createQueryBuilder("memberView")
      .where("memberView.id = :id", { id: id })
      .getOneOrFail()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "memberView", err);
      });
  }

  /**
   * @description get members where membership is setz
   * @returns {Promise<member>}
   */
  static async getByRunningMembership(): Promise<Array<member>> {
    return await dataSource
      .getRepository(member)
      .createQueryBuilder("member")
      .leftJoinAndSelect("member.memberships", "membership")
      .where("membership.end IS NULL")
      .orderBy("member.lastname")
      .addOrderBy("member.firstname")
      .addOrderBy("member.nameaffix")
      .getMany()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "member", err);
      });
  }

  /**
   * @description get newsletter by member by id
   * @param {string} id
   * @returns {Promise<member>}
   */
  static async getNewsletterById(id: string): Promise<member> {
    return await dataSource
      .getRepository(member)
      .createQueryBuilder("member")
      .leftJoinAndMapOne(
        "member.sendNewsletter",
        "member.communications",
        "sendNewsletter",
        "sendNewsletter.isSendNewsletter = true"
      )
      .where("member.id = :id", { id: id })
      .getOneOrFail()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "member", err);
      });
  }

  /**
   * @description get latest inserted memberId
   * @returns {Promise<string>}
   */
  static async getLatestInternalId(): Promise<string> {
    return await dataSource
      .getRepository(member)
      .createQueryBuilder("member")
      .where("member.internalId IS NOT NULL")
      .andWhere({ internalId: Not("") })
      .orderBy("member.createdAt", "DESC")
      .addOrderBy("member.internalId", "DESC")
      .limit(1)
      .getOne()
      .then((res) => {
        return res?.internalId ?? "";
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "memberId", err);
      });
  }

  /**
   * @description apply member joins to query
   * @returns {SelectQueryBuilder<member>}
   */
  static applyMemberBaseJoins(): SelectQueryBuilder<member> {
    return dataSource
      .getRepository(member)
      .createQueryBuilder("member")
      .leftJoinAndMapOne(
        "member.firstMembershipEntry",
        "member.memberships",
        "membership_first",
        'membership_first.memberId = member.id AND membership_first.start = (SELECT MIN("m_first"."start") FROM "membership" "m_first" WHERE "m_first"."memberId" = "member"."id")'
      )
      .leftJoinAndMapOne(
        "member.lastMembershipEntry",
        "member.memberships",
        "membership_last",
        'membership_last.memberId = member.id AND membership_last.start = (SELECT MAX("m_last"."start") FROM "membership" "m_last" WHERE "m_last"."memberId" = "member"."id")'
      )
      .leftJoinAndSelect("membership_first.status", "status_first")
      .leftJoinAndSelect("membership_last.status", "status_last")
      .leftJoinAndMapMany(
        "member.preferredCommunication",
        "member.communications",
        "preferredCommunication",
        "preferredCommunication.preferred = true"
      )
      .leftJoinAndSelect("preferredCommunication.type", "communicationtype_preferred")
      .leftJoinAndMapOne(
        "member.sendNewsletter",
        "member.communications",
        "sendNewsletter",
        "sendNewsletter.isSendNewsletter = true"
      )
      .leftJoinAndSelect("sendNewsletter.type", "communicationtype")
      .leftJoinAndMapMany(
        "member.smsAlarming",
        "member.communications",
        "smsAlarming",
        "smsAlarming.isSMSAlarming = true"
      )
      .leftJoinAndSelect("smsAlarming.type", "communicationtype_smsAlarming")
      .leftJoinAndSelect("member.salutation", "salutation");
  }
}
