import { In } from "typeorm";
import { dataSource } from "../../data-source";
import { damageReport } from "../../entity/unit/damageReport";
import DatabaseActionException from "../../exceptions/databaseActionException";

export default abstract class DamageReportService {
  private static query = () =>
    dataSource
      .getRepository(damageReport)
      .createQueryBuilder("damageReport")
      .leftJoinAndSelect("damageReport.equipment", "equipment")
      .leftJoinAndSelect("damageReport.vehicle", "vehicle")
      .leftJoinAndSelect("damageReport.wearable", "wearable")
      .leftJoinAndSelect("damageReport.repair", "repair");
  /**
   * @description get all damageReports By done
   * @returns {Promise<[Array<damageReport>, number]>}
   */
  static async getAll({
    offset = 0,
    count = 25,
    noLimit = false,
    ids = [],
  }: {
    offset?: number;
    count?: number;
    noLimit?: boolean;
    ids?: Array<string>;
  }): Promise<[Array<damageReport>, number]> {
    let query = this.query();

    if (ids.length != 0) {
      query = query.where({ id: In(ids) });
    }

    if (!noLimit) {
      query = query.offset(offset).limit(count);
    }

    return await query
      .orderBy("damageReport.reportedAt", "ASC")
      .getManyAndCount()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "damageReport", err);
      });
  }

  /**
   * @description get all damageReports By done
   * @returns {Promise<[Array<damageReport>, number]>}
   */
  static async getAllByStatus(
    done = false,
    {
      offset = 0,
      count = 25,
      noLimit = false,
    }: {
      offset?: number;
      count?: number;
      noLimit?: boolean;
    }
  ): Promise<[Array<damageReport>, number]> {
    let query = this.query().where({ done });

    if (!noLimit) {
      query = query.offset(offset).limit(count);
    }

    return await query
      .orderBy("damageReport.reportedAt", "ASC")
      .getManyAndCount()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "damageReport", err);
      });
  }

  /**
   * @description get all damageReports By related
   * @returns {Promise<[Array<damageReport>, number]>}
   */
  static async getAllForRelated(
    where: { equipmentId: string } | { vehicleId: string } | { wearableId: string },
    {
      offset = 0,
      count = 25,
      noLimit = false,
    }: {
      offset?: number;
      count?: number;
      noLimit?: boolean;
    }
  ): Promise<[Array<damageReport>, number]> {
    let query = this.query().where(where);

    if (!noLimit) {
      query = query.offset(offset).limit(count);
    }

    return await query
      .orderBy("damageReport.reportedAt", "ASC")
      .getManyAndCount()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "damageReport", err);
      });
  }

  /**
   * @description get damageReport by id
   * @returns {Promise<damageReport>}
   */
  static async getById(id: string): Promise<damageReport> {
    return await this.query()
      .where({ id })
      .getOneOrFail()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "damageReport", err);
      });
  }
}
