import { Like, In } from "typeorm";
import { dataSource } from "../../../data-source";
import { inspectionPlan } from "../../../entity/unit/inspection/inspectionPlan";
import DatabaseActionException from "../../../exceptions/databaseActionException";

export default abstract class InspectionPlanService {
  private static query = () =>
    dataSource
      .getRepository(inspectionPlan)
      .createQueryBuilder("inspectionPlan")
      .leftJoinAndMapOne(
        "inspectionPlan.latestVersionedPlan",
        "inspectionPlan.versionedPlans",
        "latestVersionedPlan",
        'latestVersionedPlan.inspectionPlanId = inspectionPlan.id AND latestVersionedPlan.version = (SELECT MAX("ivp"."version") FROM "inspection_versioned_plan" "ivp" WHERE "ivp"."inspectionPlanId" = "inspectionPlan"."id")'
      )
      .leftJoinAndSelect("latestVersionedPlan.inspectionPoints", "inspectionPoints")
      .leftJoinAndSelect("inspectionPlan.equipmentType", "equipmentType")
      .leftJoinAndSelect("inspectionPlan.vehicleType", "vehicleType")
      .leftJoinAndSelect("inspectionPlan.wearableType", "wearableType");

  /**
   * @description get all inspectionPlans for related
   * @returns {Promise<[Array<inspectionPlan>, number]>}
   */
  static async getAll({
    offset = 0,
    count = 25,
    search = "",
    noLimit = false,
    ids = [],
  }: {
    offset?: number;
    count?: number;
    search?: string;
    noLimit?: boolean;
    ids?: Array<string>;
  }): Promise<[Array<inspectionPlan>, number]> {
    let query = this.query();

    if (search != "") {
      query = query.where({
        title: Like(`%${search}%`),
      });
    }

    if (ids.length != 0) {
      query = query.where({ id: In(ids) });
    }

    if (!noLimit) {
      query = query.offset(offset).limit(count);
    }

    return await query
      .orderBy("inspectionPlan.title", "ASC")
      .getManyAndCount()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "inspectionPlan", err);
      });
  }

  /**
   * @description get all inspectionPlans for related
   * @returns {Promise<[Array<inspectionPlan>, number]>}
   */
  static async getAllForRelated(
    where: { equipmentTypeId: string } | { vehicleTypeId: string } | { wearableTypeId: string },
    {
      offset = 0,
      count = 25,
      search = "",
      noLimit = false,
      ids = [],
    }: {
      offset?: number;
      count?: number;
      search?: string;
      noLimit?: boolean;
      ids?: Array<string>;
    }
  ): Promise<[Array<inspectionPlan>, number]> {
    let query = this.query().where(where);

    if (search != "") {
      query = query.andWhere({
        title: Like(`%${search}%`),
      });
    }

    if (ids.length != 0) {
      query = query.andWhere({ id: In(ids) });
    }

    if (!noLimit) {
      query = query.offset(offset).limit(count);
    }

    return await query
      .orderBy("inspectionPlan.title", "ASC")
      .getManyAndCount()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "inspectionPlan", err);
      });
  }

  /**
   * @description get inspectionPlan by id
   * @returns {Promise<inspectionPlan>}
   */
  static async getById(id: string): Promise<inspectionPlan> {
    return await this.query()
      .where({ id })
      .getOneOrFail()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "inspectionPlan", err);
      });
  }
}
