import { IsNull, Not } from "typeorm";
import { dataSource } from "../../../data-source";
import { inspection } from "../../../entity/unit/inspection/inspection";
import DatabaseActionException from "../../../exceptions/databaseActionException";

export default abstract class InspectionService {
  private static query = () =>
    dataSource
      .getRepository(inspection)
      .createQueryBuilder("inspection")
      .leftJoinAndSelect("inspection.inspectionPlan", "inspectionPlan")
      .leftJoinAndSelect("inspectionPlan.equipmentType", "equipmentType")
      .leftJoinAndSelect("inspectionPlan.vehicleType", "vehicleType")
      .leftJoinAndSelect("inspectionPlan.wearableType", "wearableType")
      .leftJoinAndSelect("inspection.inspectionVersionedPlan", "inspectionVersionedPlan")
      .leftJoinAndSelect("inspectionVersionedPlan.inspectionPoints", "inspectionPoints")
      .leftJoinAndSelect("inspection.pointResults", "pointResults")
      .leftJoinAndSelect("pointResults.inspectionPoint", "inspectionPoint")
      .leftJoinAndSelect("inspection.equipment", "equipment")
      .leftJoinAndSelect("inspection.vehicle", "vehicle")
      .leftJoinAndSelect("inspection.wearable", "wearable");

  private static minifiedQuery = () =>
    dataSource
      .getRepository(inspection)
      .createQueryBuilder("inspection")
      .leftJoinAndSelect("inspection.inspectionPlan", "inspectionPlan")
      .leftJoinAndSelect("inspectionPlan.equipmentType", "equipmentType")
      .leftJoinAndSelect("inspectionPlan.vehicleType", "vehicleType")
      .leftJoinAndSelect("inspectionPlan.wearableType", "wearableType")
      .leftJoinAndSelect("inspection.equipment", "equipment")
      .leftJoinAndSelect("inspection.vehicle", "vehicle")
      .leftJoinAndSelect("inspection.wearable", "wearable");

  /**
   * @description get all inspections sorted by next inspection not having newer
   * @returns {Promise<Array<inspection>>}
   */
  static async getAllSortedNotHavingNewer({
    offset = 0,
    count = 25,
    noLimit = false,
  }: {
    offset?: number;
    count?: number;
    noLimit?: boolean;
  }): Promise<[Array<inspection>, number]> {
    let query = this.query().where({ hasNewer: false });

    if (!noLimit) {
      query = query.offset(offset).limit(count);
    }

    return await query
      .orderBy("inspection.nextInspection", "ASC", "NULLS LAST")
      .getManyAndCount()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "inspection", err);
      });
  }

  /**
   * @description get all inspections running
   * @returns {Promise<Array<inspection>>}
   */
  static async getAllRunning({
    offset = 0,
    count = 25,
    noLimit = false,
  }: {
    offset?: number;
    count?: number;
    noLimit?: boolean;
  }): Promise<[Array<inspection>, number]> {
    let query = this.minifiedQuery().where({ finishedAt: IsNull() });

    if (!noLimit) {
      query = query.offset(offset).limit(count);
    }

    return await query
      .orderBy("inspection.createdAt", "ASC")
      .getManyAndCount()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "inspection", err);
      });
  }

  /**
   * @description get all inspections for related
   * @returns {Promise<Array<inspection>>}
   */
  static async getAllForRelated(
    where: { equipmentId: string } | { vehicleId: string } | { wearableId: string },
    {
      offset = 0,
      count = 25,
      noLimit = false,
    }: {
      offset?: number;
      count?: number;
      noLimit?: boolean;
    }
  ): Promise<[Array<inspection>, number]> {
    let query = this.minifiedQuery().where(where);

    if (!noLimit) {
      query = query.offset(offset).limit(count);
    }

    return await query
      .orderBy("inspection.createdAt", "DESC")
      .getManyAndCount()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "inspection", err);
      });
  }

  /**
   * @description get inspection by id
   * @returns {Promise<inspection>}
   */
  static async getById(id: string): Promise<inspection> {
    return await this.query()
      .where({ id })
      .getOneOrFail()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "inspection", err);
      });
  }

  /**
   * @description uses versionedPlan
   * @returns {Promise<boolean>}
   */
  static async existsUnfinishedInspectionToPlan(
    inspectionPlanId: string,
    relation: "vehicle" | "equipment" | "wearable",
    relationId: string
  ): Promise<boolean> {
    let where: { equipmentId: string } | { vehicleId: string } | { wearableId: string };
    if (relation == "equipment") {
      where = { equipmentId: relationId };
    } else if (relation == "vehicle") {
      where = { vehicleId: relationId };
    } else {
      where = { wearableId: relationId };
    }
    return await dataSource
      .getRepository(inspection)
      .createQueryBuilder("inspection")
      .where({ inspectionPlanId, finishedAt: IsNull(), ...where })
      .getExists()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "used inspection", err);
      });
  }

  /**
   * @description uses versionedPlan
   * @returns {Promise<boolean>}
   */
  static async usesVersionedInspectionPlan(inspectionVersionedPlanId: string): Promise<boolean> {
    return await dataSource
      .getRepository(inspection)
      .createQueryBuilder("inspection")
      .where({ inspectionVersionedPlanId })
      .getExists()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "used inspection", err);
      });
  }
}
