import { IsNull, Not } from "typeorm";
import { dataSource } from "../../data-source";
import { repair } from "../../entity/unit/repair";
import DatabaseActionException from "../../exceptions/databaseActionException";

export default abstract class RepairService {
  private static query = () =>
    dataSource
      .getRepository(repair)
      .createQueryBuilder("repair")
      .leftJoinAndSelect("repair.equipment", "equipment")
      .leftJoinAndSelect("repair.vehicle", "vehicle")
      .leftJoinAndSelect("repair.wearable", "wearable")
      .leftJoinAndSelect("repair.reports", "reports");

  /**
   * @description get all repairs
   * @returns {Promise<[Array<repair>, number]>}
   */
  static async getAll(
    done = false,
    {
      offset = 0,
      count = 25,
      noLimit = false,
    }: {
      offset?: number;
      count?: number;
      noLimit?: boolean;
    }
  ): Promise<[Array<repair>, number]> {
    let query = this.query().where({ finishedAt: done ? Not(IsNull()) : IsNull() });
    if (!noLimit) {
      query = query.offset(offset).limit(count);
    }

    return await query
      .orderBy("repair.createdAt", "ASC")
      .getManyAndCount()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "repair", err);
      });
  }

  /**
   * @description get all repairs By related
   * @returns {Promise<[Array<repair>, number]>}
   */
  static async getAllForRelated(
    where: { equipmentId: string } | { vehicleId: string } | { wearableId: string },
    {
      offset = 0,
      count = 25,
      noLimit = false,
    }: {
      offset?: number;
      count?: number;
      noLimit?: boolean;
    }
  ): Promise<[Array<repair>, number]> {
    let query = this.query().where(where);

    if (!noLimit) {
      query = query.offset(offset).limit(count);
    }

    return await query
      .orderBy("repair.createdAt", "ASC")
      .getManyAndCount()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "repair", err);
      });
  }

  /**
   * @description get repair by id
   * @returns {Promise<repair>}
   */
  static async getById(id: string): Promise<repair> {
    return await this.query()
      .where({ id })
      .getOneOrFail()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "repair", err);
      });
  }
}
