import { Like, In } from "typeorm";
import { dataSource } from "../../../data-source";
import { vehicle } from "../../../entity/unit/vehicle/vehicle";
import DatabaseActionException from "../../../exceptions/databaseActionException";

export default abstract class VehicleService {
  /**
   * @description get all vehicles
   * @returns {Promise<[Array<vehicle>, number]>}
   */
  static async getAll({
    offset = 0,
    count = 25,
    search = "",
    noLimit = false,
    ids = [],
  }: {
    offset?: number;
    count?: number;
    search?: string;
    noLimit?: boolean;
    ids?: Array<string>;
  }): Promise<[Array<vehicle>, number]> {
    let query = dataSource
      .getRepository(vehicle)
      .createQueryBuilder("vehicle")
      .leftJoinAndSelect("vehicle.vehicleType", "vehicletype");

    if (search != "") {
      query = query
        .where({
          code: Like(`%${search}%`),
        })
        .orWhere({
          name: Like(`%${search}%`),
        })
        .orWhere({
          location: Like(`%${search}%`),
        });
    }

    if (ids.length != 0) {
      query = query.where({ id: In(ids) });
    }

    if (!noLimit) {
      query = query.offset(offset).limit(count);
    }

    return await query
      .orderBy("name", "ASC")
      .getManyAndCount()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "vehicle", err);
      });
  }

  /**
   * @description get vehicle by code
   * @returns {Promise<Array<vehicle>>}
   */
  static async getAllByCode(code: string): Promise<Array<vehicle>> {
    return await dataSource
      .getRepository(vehicle)
      .createQueryBuilder("vehicle")
      .leftJoinAndSelect("vehicle.vehicleType", "vehicletype")
      .where({ code: Like(`%${code}%`) })
      .orderBy("name", "ASC")
      .getMany()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "vehicle", err);
      });
  }

  /**
   * @description get vehicle by id
   * @returns {Promise<vehicle>}
   */
  static async getById(id: string): Promise<vehicle> {
    return await dataSource
      .getRepository(vehicle)
      .createQueryBuilder("vehicle")
      .leftJoinAndSelect("vehicle.vehicleType", "vehicletype")
      .where({ id })
      .getOneOrFail()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "vehicle", err);
      });
  }
}
