import { Like, In } from "typeorm";
import { dataSource } from "../../../data-source";
import { wearable } from "../../../entity/unit/wearable/wearable";
import DatabaseActionException from "../../../exceptions/databaseActionException";

export default abstract class WearableService {
  /**
   * @description get all wearables
   * @returns {Promise<[Array<wearable>, number]>}
   */
  static async getAll({
    offset = 0,
    count = 25,
    search = "",
    noLimit = false,
    ids = [],
  }: {
    offset?: number;
    count?: number;
    search?: string;
    noLimit?: boolean;
    ids?: Array<string>;
  }): Promise<[Array<wearable>, number]> {
    let query = dataSource
      .getRepository(wearable)
      .createQueryBuilder("wearable")
      .leftJoinAndSelect("wearable.wearableType", "wearabletype")
      .leftJoinAndSelect("wearable.wearer", "wearer");

    if (search != "") {
      query = query
        .where({
          code: Like(`%${search}%`),
        })
        .orWhere({
          name: Like(`%${search}%`),
        })
        .orWhere({
          location: Like(`%${search}%`),
        });
    }

    if (ids.length != 0) {
      query = query.where({ id: In(ids) });
    }

    if (!noLimit) {
      query = query.offset(offset).limit(count);
    }

    return await query
      .orderBy("name", "ASC")
      .getManyAndCount()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "wearable", err);
      });
  }

  /**
   * @description get wearable by code
   * @returns {Promise<Array<wearable>>}
   */
  static async getAllByCode(code: string): Promise<Array<wearable>> {
    return await dataSource
      .getRepository(wearable)
      .createQueryBuilder("wearable")
      .leftJoinAndSelect("wearable.wearableType", "wearabletype")
      .where({ code: Like(`%${code}%`) })
      .orderBy("name", "ASC")
      .getMany()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "wearable", err);
      });
  }

  /**
   * @description get wearable by id
   * @returns {Promise<wearable>}
   */
  static async getById(id: string): Promise<wearable> {
    return await dataSource
      .getRepository(wearable)
      .createQueryBuilder("wearable")
      .leftJoinAndSelect("wearable.wearableType", "wearabletype")
      .leftJoinAndSelect("wearable.wearer", "wearer")
      .where({ id })
      .getOneOrFail()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "wearable", err);
      });
  }
}
